# encoding: UTF-8
class AutomatizacionObserver < ActiveRecord::Observer
  observe :gasto, :presupuesto, :actividad, :indicador, :fuente_verificacion, :documento, :presupuesto_ingreso, :ingreso, :contrato

  # Genera comentarios y marcado cuando el objeto se modifica
  def after_save(objeto)
    # Esto tendremos que cambiarlo cuando mezclemos comentarios y marcado para evitar que haya anotaciones dobles (comentario + marcado => comentario)
    # Si ha cambiado y no ha sido solo el marcado o la fecha de actualizacion...
    cambios = objeto.changed
    cambios.delete("marcado_id") if objeto.respond_to?('marcado_id')
    cambios.delete("updated_at") if objeto.respond_to?('updated_at')
    if cambios.size > 0
      elemento = elemento_relacionado objeto
      marcar_elemento elemento if elemento.respond_to?('marcado')
      comentario_elemento(elemento,objeto,cambios) if UserInfo.current_user && elemento.respond_to?('comentario')
    end
  end

  # Genera comentarios y marcado cuando un objeto hijo se elimina
  # Revisar que funcione ok
  #def after_destroy(objeto)
  #  elemento = elemento_relacionado objeto
  #  unless elemento.destroyed?
  #    comentario_elemento(elemento,objeto,[]) if UserInfo.current_user && elemento.respond_to?('comentario')
  #  end
  #end

  # Marca el elemento cuando se ha modificado o creado
  def marcar_elemento(objeto)
    # Marca el elemento solo si no hemos hecho cambios en el marcado
    if objeto.marcado_id == objeto.marcado_id_was
      marc=Marcado.find_by_automatico(true)
      if marc
        # Guarda evitando callbacks 
        objeto.update_column(:marcado_id, marc.id)
      end
    end
  end

  # Genera un comentario automatico con la modificacion
  def comentario_elemento(elemento, objeto, cambios)
    nombre_objeto = objeto.class.name.underscore.humanize
    if objeto.destroyed?
      texto = nombre_objeto + " " + _("eliminado")
    elsif objeto.id_was.nil?
      # Comentario de que el elemento ha sido creado
      texto = nombre_objeto + " " + _("creado")
    else
      # Comentario de elemento modificado (junto a los campos tocados)
      texto = nombre_objeto + " " + _("modificado") + " (" + _("campos") + ": " + cambios.join(", ") + ") "
    end
    elemento.comentario << Comentario.create(:usuario_id => UserInfo.current_user.id, :texto => texto)
  end

 private

  # Devuelve los elementos que recogen la modificacion del objeto
  def elemento_relacionado(objeto)
    case objeto.class.name
      when "EstadoContrato" then objeto.contrato
      when "Pago" then objeto.gasto
      else objeto
    end 
  end

end

